<?php

class modelIbackuper extends cmsModel {

    public function getCtype($id) {

        $content_model = cmsCore::getModel('content');

        $ctype = $content_model->getContentType($id);

        if (!$ctype) { return false; }

        $datasets = $content_model->getContentDatasets($id);

        $cats = $content_model->getCategoriesTree($ctype['name'], false);

        $fields = $content_model->getContentFields($ctype['name']);

        foreach ($fields as $name => $field){
            unset($fields[$name]['handler_title'], $fields[$name]['handler']);
        }

        $props = $content_model->getContentProps($ctype['name']);

        $props_bind = $this->getPropsBinds($ctype['name']);

        $moderators = $content_model->getContentTypeModerators($ctype['name']);

        $relations = $content_model->getContentRelations($id);

        $rules  = cmsPermissions::getRulesList('content');

        $perm_values = cmsPermissions::getPermissions($ctype['name']);

        $sitemap_options = cmsController::loadOptions('sitemap');

        $images_presets = cmsCore::getModel('images')->get('images_presets', function($item, $model){
            unset($item['id']);
            $item['wm_image'] = cmsModel::yamlToArray($item['wm_image']);
            return $item;
        }, 'name');

        return array(
            'ctype' => $ctype,
            'cats' => $cats,
            'fields' => $fields,
            'props' => $props,
            'props_bind' => $props_bind,
            'datasets' => $datasets,
            'moderators' => $moderators,
            'relations' => $relations,
            'rules' => $rules,
            'perm_values' => $perm_values,
            'sitemap_options' => !empty($sitemap_options['sources']['[content|' . $ctype['name'] . ']']) ? true : false,
            'images_presets' => $images_presets
        );
    }

    public function addCtypeFields($ctype_name, $fields) {

        $content_model = cmsCore::getModel('content');
        $content_table_name = $content_model->table_prefix . $ctype_name;
        $fields_table_name = $content_model->table_prefix . $ctype_name . '_fields';

        $sql = "TRUNCATE TABLE {#}{$fields_table_name}";
        $content_model->db->query($sql);

        $content_model->db->dropTableField($content_table_name, 'content');
        $content_model->db->dropTableField($content_table_name, 'photo');

        foreach ($fields as $name => $field){

            if (!in_array($name, array('title', 'user', 'date_pub'))){
                $field_class = "field" . string_to_camel('_', $field['type']);
                $field_parser = new $field_class(null, null);

                $sql = "ALTER TABLE {#}{$content_table_name} ADD `{$field['name']}` {$field_parser->getSQL()}";
                $content_model->db->query($sql);

                if($field_parser->is_denormalization){
                    $cfield_name = $field['name'].cmsFormField::FIELD_CACHE_POSTFIX;
                    $sql = "ALTER TABLE {#}{$content_table_name} ADD `{$cfield_name}` {$field_parser->getCacheSQL()}";
                    $content_model->db->query($sql);
                }

                if ($field['is_in_filter'] && $field_parser->allow_index){
                    $sql = "ALTER TABLE `{#}{$content_table_name}` ADD INDEX ( `{$field['name']}` )";
                    $content_model->db->query($sql);
                }
            }

            $content_model->db->insert($fields_table_name, $field);

            cmsEventsManager::hook('ctype_field_after_add', array($field, $ctype_name, $content_model));

            // если есть опция полнотекстового поиска
            if(is_array($field['options']) && !empty($field['options']['in_fulltext_search'])){
                // получаем полнотекстовый индекс для таблицы, он может быть только один
                $fulltext_index = $content_model->db->getTableIndexes($content_table_name, 'FULLTEXT');
                if($fulltext_index){
                    // название индекса
                    $index_name = key($fulltext_index);
                    // поля индекса
                    $index_fields = $fulltext_index[$index_name];
                    // ищем, нет ли такого поля уже в индексе, мало ли :-)
                    $key = array_search($field['name'], $index_fields);
                    // не нашли, добавляем
                    if($key === false){
                        // удаляем старый индекс
                        $content_model->db->dropIndex($content_table_name, $index_name);
                        // создаем новый
                        $content_model->createFullTextIndex($ctype_name, $field['name']);
                    }
                } else {
                    $content_model->createFullTextIndex($ctype_name, $field['name']);
                }
            }

        }

        return true;
    }

    public function getPropsBinds($ctype_name, $category_id=false){

        $this->selectOnly('p.*');
        $this->select('p.id', 'prop_id');
        $this->select('i.*');

        $this->join('con_' . $ctype_name . '_props', 'p', 'p.id = i.prop_id');

        if ($category_id){
            $this->filterEqual('cat_id', $category_id);
        }

        $this->orderBy('ordering');

        return $this->get('con_' . $ctype_name . '_props_bind', function($item){
            $item['options'] = cmsModel::yamlToArray($item['options']);
            return $item;
        });
    }

    public function getFormIforms($id) {

        $iforms_model = cmsCore::getModel('iforms');

        $form = $iforms_model->getFormItemsById($id);

        if (!$form) { return false; }

        unset($form['frequency']);

        $fields = $iforms_model->getFormFields('iforms_fields', $form['id'], 'iforms_id');

        foreach ($fields as $name => $field){
            unset($fields[$name]['handler_title'], $fields[$name]['handler']);
        }

        return array(
            'form' => $form,
            'fields' => $fields
        );

    }

}